<!DOCTYPE style-sheet PUBLIC "-//James Clark//DTD DSSSL Style
Sheet//EN" [

<!ENTITY isoprocs PUBLIC "ISO/IEC 10179:1996//NONSGML
Procedures::Reference Implementations//EN" NDATA DSSSL>

<!ENTITY mifout PUBLIC "-//O'Reilly//NONSGML Procedures::MIF
Generation Procedures//EN" NDATA DSSSL>

<!ENTITY useful PUBLIC "-//O'Reilly//NONSGML Procedures::Useful DSSSL
Procedures//EN" NDATA DSSSL>

<!ENTITY amp CDATA "&">
<!ENTITY lt CDATA "<">
]>

<style-specification id="db2mif"
use="iso-10179-procedures mifout useful">

;; Refer to this stylesheet as
;; PUBLIC
;;    "-//O'Reilly//NONSGML Stylesheet::DocBook 3.0 to MIF 5.0//EN"
;; Your stylesheet must define the variable (element-to-process),
;;    which must be a singleton node list.
;; 
;; Use this stylesheet with a valid DocBook 3.0 document instance.
;; It currently only handles a small set of elements.
;; After using it, open the resulting MIF and import the O'Reilly
;;   template.
;;
;; known limitations:
;; Doesn't know what the current PfgFmt is when handling in-line
;;    elements
;; Ignores index entries

;; blocks is a list of GIs of elements which create their own MIF
;; paragraphs
(define blocks
  '("BLOCKQUOTE"
    "INFORMALTABLE"
    "ITEMIZEDLIST"
    "LITERALLAYOUT"
    "NOTE"
    "ORDEREDLIST"
    "PARA"
    "PROGRAMLISTING"
    "SCREEN"
    "SIDEBAR"
    "SIMPLELIST"
    "TABLE"
    "TITLE"
    "VARIABLELIST"
    "WARNING"))

(define (gen-file-name)
  (string-append (if (equal? (gi (current-node))
			     "CHAPTER")
		     (string-append "ch-"
				    (number->string (element-number (current-node))))
		   (if (ancestor "CHAPTER")
		       (string-append "ch-"
				      (number->string (element-number (ancestor "CHAPTER"))))
		     (if (or (equal? (gi (current-node))
				     "PREFACE")
			     (ancestor "PREFACE"))
			 "preface"
		       (if (or (equal? (gi (current-node))
				       "GLOSSARY")
			       (ancestor "GLOSSARY"))
			   "glossary"
			 (error "Unknown Container!")))))
		 ".mif"))

;; inlines is a list of GIs of elements after which the current text
;; style (FTag) needs to be restored
(define inlines
  '("ACRONYM"
    "EMPHASIS"
    "LITERAL"
    "REPLACEABLE"
    "SYSTEMITEM"
    "USERINPUT"))

;; (make-columns) creates MIF column definitions for a CALS table.
;; The default is a variable column of 0-2 inches.
(define (make-columns)
  (if (equal? (gi (current-node))
		  "SIDEBAR")
      (make formatting-instruction
	    data: "   &lt;TblColumn 
    &lt;TblColumnNum 0>
    &lt;TblColumnWidthP 1>
   > # end of TblColumn
")
      (let col-loop ((num-columns (string->number (attribute-string "cols"
								    (get-children-by-type '("TGROUP")
											  (current-node)))))
		     (this-column 0)
		     (colspecs (get-children-by-type '("COLSPEC")
						     (get-children-by-type '("TGROUP")
									   (current-node)))))
	   (sosofo-append (make formatting-instruction
				data: (string-append "   &lt;TblColumn 
    &lt;TblColumnNum "
						     (number->string this-column)
						     ">
"
						     (if (equal? (attribute-string "colnum"
										   (node-list-first colspecs))
								 (number->string (+ this-column 1)))
							 (let ((colwidth (attribute-string "colwidth"
											   (node-list-first colspecs))))
							   (if colwidth
							       (string-append "    &lt;TblColumnWidth "
									      (substring colwidth
											 0
											 (- (string-length colwidth) 1))
									      "\">
")
							       "    &lt;TblColumnWidthA 0 2\">
"))
						         "    &lt;TblColumnWidthA 0 2\">
")
						     "   > # end of TblColumn
"))
			  (if (equal? num-columns
				      (+ this-column 1))
			      (empty-sosofo)
			      (col-loop num-columns
					(+ this-column 1)
					(if (equal? (attribute-string "colnum"
								      (node-list-first colspecs))
						    (number->string (+ this-column 1)))
					    (node-list-rest colspecs)
					    colspecs)))))))

;; (make-footnotes) is a procedure to find the footnote children of a
;; node
(define (make-footnotes)
  (with-mode footnotes
	     (process-node-list (node-list-filter (lambda (snl)
						    (equal? (node-property 'class-name
									   snl)
							    'element))
						  (children (current-node))))))

;; (make-table-content) process the children of the table in the order
;; MIF wants them.
(define (make-table-content)
  (if (equal? (gi (current-node))
		  "SIDEBAR")
      (make-table-cell "SidebarBody")
      (sosofo-append (process-first-descendant "TITLE")
		     (process-first-descendant "THEAD")
		     (process-first-descendant "TBODY")
		     (process-first-descendant "TFOOT"))))

;; nobreaks is a list of MIF FTags for which line-break PIs should be
;; ignored - notably in cross-references to broken titles
(define nobreaks
  (append mif-nobreaks
	  '("ChapterTitle")))

;; (pi-handler this-node parafmt stringfmt) handles processing
;; instructions from (process-text)
(define (pi-handler this-node
		    parafmt
		    stringfmt)
  (case (node-property 'system-data
		       this-node)
	(("lb") (if (member stringfmt
			    nobreaks)
		    (make formatting-instruction
			  data: " ")
		    (line-break)))
	(else (empty-sosofo))))

;; preserves is a list of GIs for which a new paragraph should be
;; created at each line break
(define preserves
  '("LITERALLAYOUT"
    "PROGRAMLISTING"
    "SCREEN"))

;; (sdata-handler this-node parafmt stringfmt) handles SDATA entities
;; from (process-text).
(define (sdata-handler this-node
		       parafmt
		       stringfmt)
  (case (node-property 'system-data
		       this-node)
	;; ampersand
	(("[amp   ]") (make formatting-instruction
			    data: "&amp;"))
	;; back-quote
	(("[bquot]") (make formatting-instruction
			   data: "\\x60 "))
	;; back-slash
	(("[bsol  ]") (make formatting-instruction
			    data: "\\\\"))
	;; em space: (mif-char "EmSpace")
	(("[emsp  ]") (make formatting-instruction
			    data: "\\x14 "))
	;; en space: (mif-char "EnSpace")
	(("[ensp  ]") (make formatting-instruction
			    data: "\\x13 "))
	;; e with umlaut
	(("[euml  ]") (make formatting-instruction
			    data: "\\x91 "))
	;; greater-than sign
	(("[gt    ]") (make formatting-instruction
			    data: "\\>"))
	;; hair space: (mif-char "ThinSpace")
	(("[hairsp]") (make formatting-instruction
			    data: "\\x12 "))
	;; horizontal ellipses
	(("[hellip]") (make formatting-instruction
			    data: "\\xc9 "))
	;; i with umlaut
	(("[iuml  ]") (make formatting-instruction
			    data: "\\x95 "))
	;; left double quote
	(("[ldquo ]") (make formatting-instruction
			    data: "\\xd2 "))
	;; left single quote
	(("[lsquo ]") (make formatting-instruction
			    data: "\\xd4 "))
	;; less-than sign
	(("[lt    ]") (make formatting-instruction
			    data: "&lt;"))
	;; em dash: (var-ref "CharEmDash")
	(("[mdash ]" "[mdash]") (make formatting-instruction
				      data: "\\xd1 "))
	;; non-breaking space: (var-ref "SpaceNonBreaking")
	(("[nbsp  ]") (make formatting-instruction
			    data: "\\x11 "))
	;; en dash: (var-ref "CharEnDash")
	(("[ndash ]" "[ndash]") (make formatting-instruction
				      data: "\\xd0 "))
	;; number space: (mif-char "NumberSpace")
	(("[numsp ]") (make formatting-instruction
			    data: "\\x10 "))
	;; no-width space
	(("[nwsp]") (empty-sosofo))
	;; pi: (var-ref "MathPi")
	(("[pi    ]") (if (equal? stringfmt
				  "text-only")
			  (make formatting-instruction
				data: "[pi]")
			  (var-ref "MathPi")))
	;; right arrow: (var-ref "CharArrowRight")
	(("[rarr  ]") (if (equal? stringfmt
				  "text-only")
			  (make formatting-instruction
				data: "->")
			  (var-ref "CharArrowRight")))
	;; right double quote
	(("[rdquo ]") (make formatting-instruction
			    data: "\\xd3 "))
	;; right single quote
	(("[rsquo ]") (make formatting-instruction
			    data: "\\xd5 "))
	;; thin space: (var-ref "SpaceThin")
	(("[thinsp]") (make formatting-instruction
			    data: "\\x12 "))
	;; double-quote
	(("[quot]") (make formatting-instruction
			  data: "\""))))

;; table-formats is a list of TblFormats used in the document; without
;; this, Frame will crash
(define table-formats
  '("Sidebar" "TableNoTitle" "TableWithTitle"))

;; xref-formats is a list of XRefFormats used in the document
(define xref-formats
  '("ChapTitle" "Head"))

;; process footnotes in MIF header
(mode footnotes
      (default (empty-sosofo))

      (element chapter
	       (process-children))

      (element footnote
	       (make-footnote))

      (element (footnote para)
	       (with-mode #f
			  (make-para (if (node-list-empty? (ancestor "TGROUP"))
					 "Footnote"
				         "TableFootnote")
				     #f)))

      (element para
	       (process-node-list (get-children-by-type '("FOOTNOTE")
							(current-node))))

      (element sect1
	       (process-children))

      (element sect2
	       (process-children))

      (element sect3
	       (process-children)))

;; process tables in MIF header
(mode tables
      (element chapter
	       (process-children))

      (element entry
	       (make-table-cell (if (node-list-empty? (ancestor "TBODY"))
				    "CellHeading"
				    "CellBody")))

      (element informaltable
	       (make-table "TableNoTitle"
			   (attribute-string "cols"
					     (get-children-by-type '("TGROUP")
								   (current-node)))))

      (element (sidebar literallayout)
	       (with-mode #f
			  (make-para "SidebarListSimple"
				     #f)))

      (element (entry para)
	       (with-mode #f
			  (make-para (if (node-list-empty? (ancestor "TBODY"))
					 "CellHeading"
				         "CellBody")
				     #f)))

      (element row
	       (make-table-row))

      (element (sidebar screen)
	       (with-mode #f
			  (make-para "SidebarCode"
				     #f)))

      (element sect1
	       (process-children))

      (element sect2
	       (process-children))

      (element sect3
	       (process-children))

      (element sidebar
	       (make-table "Sidebar"
			   "1"))

      (element table
	       (make-table "TableWithTitle"
			   (attribute-string "cols"
					     (get-children-by-type '("TGROUP")
								   (current-node)))))

      (element tbody
	       (make-table-body))

      (element tfoot
	       (make-table-foot))

      (element tgroup
	       (process-children))

      (element thead
	       (make-table-head))

      (element (sidebar title)
	       (with-mode #f
			  (make-para "SidebarTitle"
				     #t)))

      (element (table title)
	       (sosofo-append (make formatting-instruction
				    data: "  &lt;TblTitle 
   &lt;TblTitleContent 
    &lt;Notes 
")
			      (make-footnotes)
			      (make formatting-instruction
				    data: (string-append "    > # end of Notes
"
							 (parafmt "TableLabel")
							 " > # end of Para
"))
			      (with-mode #f
					 (make-para "TableTitle"
						    #t))
			      (make formatting-instruction
				    data: "   > # end of TblTitleContent
  > # end of TblTitle
"))))

;; mode for processing targets of cross-references
(mode xref
      (element chapter
	       (sosofo-append (make-start-xref "ChapTitle" "ChapterTitle" "TITLE")
			      (make formatting-instruction
				    data: (string-append "   &lt;String `Chap'>
   &lt;Char NoHyphen>
   &lt;String `ter'>
   &lt;Char HardSpace>
   &lt;String `"
							 (number->string (element-number (current-node)))
							 ", '>
   &lt;Font 
    &lt;FTag `ChapterTitle'>
    &lt;FLocked No>
   > # end of Font
   &lt;String `"))
			      (process-first-descendant "TITLE")))

      (element sect2
	       (sosofo-append (make-start-xref "Head" "BHead" "TITLE")
			      (make formatting-instruction
				    data: "   &lt;String `\\xd2 '>
   &lt;String `")
			      (process-first-descendant "TITLE")
			      (make formatting-instruction
				    data: "'>
   &lt;String `\\xd3 '>
   &lt;String `")))

      (element table
	       (sosofo-append (make-start-xref "Item" "TableTitle" "TITLE")
			      (make formatting-instruction
				    data: (string-append "   &lt;String `Ta'>
   &lt;Char NoHyphen>
   &lt;String `ble'>
   &lt;String HardSpace>
   &lt;String `"
							 (number->string (element-number (ancestor "CHAPTER")))
							 "'>
   &lt;Char EnDash>
   &lt;String `"
							 (number->string (list-ref (element-number-list '("CHAPTER" "TITLE"))
										   1))))))

      (element (chapter title)
	       (with-mode #f
			  (process-text "" "ChapterTitle")))

      (element (sect2 title)
	       (with-mode #f
			  (process-text "" ""))))

(mode xref-cmd
      (element sect2
	       (sosofo-append (make-start-xref "Head" "BHead" "TITLE")
			      (make formatting-instruction
				    data: "   &lt;Font
    &lt;FTag `Command'>
    &lt;FLocked No>
   > # end of Font
   &lt;String `")
			      (process-first-descendant "TITLE")))

      (element (sect2 title)
	       (with-mode #f
			  (process-text "" "Command"))))


;; root:
(root (process-node-list (element-to-process)))

;; acronym
(element acronym
	 (make-string "" "Acronym"))

;; chapter: insert required paragraphs at chapter head, then process
;; children
(element chapter
	 (make entity
	       system-id: (gen-file-name)
	       (make-mif-start)
	       (make formatting-instruction
		     data: (string-append
			    (parafmt "ChapterStart")
			    "  &lt;PgfNumString `Chap\\x05 ter\\x11 "
			    (number->string (element-number (current-node)))
			    "'>
 > # end of Para
"
			    (parafmt "ChapterLabel")
			    "  &lt;PgfNumString `"
			    (number->string (element-number (current-node)))
			    "'>
 > # end of Para
"))
	       (process-children)
	       (make-mif-end)))

;; emphasis - usually Emphasis, but check the role attribute
(element (term emphasis)
	 (case (attribute-string "role"
				 (current-node))
	       (("bold") (make-string "" "Bold"))
	       ((#f) (process-text "ListVariableTerm" ""))))
(element emphasis
	 (make-string "" (case (attribute-string "role"
						 (current-node))
			       (("bold") "Bold")
			       (("cmd") "Command")
			       ((#f) "Emphasis"))))

;; footnotes - only make a reference in initial mode
(element footnote
	 (make-footnote-ref (current-node)))

;; footnoteref - make a reference to footnote at linkend
(element footnoteref
	 (make-footnote-ref (element-with-id (attribute-string "linkend"
							       (current-node)))))

;; glossary - create a new file eventually, but do nothing for now.
(element glossary
	 (empty-sosofo))

;; indexterm - ignore it for now
(element indexterm
	 (empty-sosofo))

;; informaltable - only make a reference in initial mode
(element informaltable
	 (sosofo-append (make formatting-instruction
			      data: (string-append (parafmt "TableHolder")
						   "  &lt;ParaLine 
"))
			(make-table-ref)
			(make formatting-instruction
			      data: "  >
 > # end of Para
")))

;; literal
(element literal
	 (make-string "" "Literal"))

;; literallayout - each line becomes a ListSimple
(element literallayout
	 (make-para "ListSimple"
		    #f))

;; member - member of a simplelist
(element member
	 (make-para "ListSimple"
		    #f))

;; para - we have lots of kinds!
(element (blockquote para)
	 (make-para "Quote"
		    #f))
(element (chapter para)
	 (make-para (if (first-sibling?)
			"BodyStart"
		        "Body")
		    #f))
(element (listitem itemizedlist listitem para)
	 (make-para (if (first-sibling?)
			"\\>ListBullet"
		        "\\>ListBullet...")
		    #f))
(element (itemizedlist listitem para)
	 (make-para (if (first-sibling?)
			"ListBullet"
		        "ListBullet...")
		    #f))
(element (note para)
	 (make-ruled-para (if (first-sibling?)
			      "Note"
			      "Note...")))
(element (orderedlist listitem para)
	 (make-para (if (first-sibling?)
			(if (first-sibling? (ancestor "LISTITEM"))
			    "ListNumberStart"
			    "ListNumber")
		        "ListNumber...")
		    #f))
(element (varlistentry listitem para)
	 (make-para "ListVariable"
		    #f))
(element (warning para)
	 (make-ruled-para (if (first-sibling?)
			      "NoteWarning"
			      "Note...")))
(element para
	 (make-para "Body"
		    #f))

;; preface - create a new file eventually, but do nothing for now.
(element preface
	 (empty-sosofo))

;; programlisting - becomes multiple paras at fixed-width
(element (entry para programlisting)
	 (make-sub-para "CellBodyCode" "CellBody"))
(element (listitem programlisting)
	 (make-para "CodeIndent"
		    #f))
(element (para programlisting)
	 (make-sub-para "CodeIndent" "Body"))
(element (warning programlisting)
	 (make-para "!Note\\>Code"
		    #f))
(element programlisting
	 (make-para "Code"
		    #f))

;; replaceable
(element replaceable
	 (make-string "" "Replaceable"))

;; screen - like programlisting (needs handling in (new-line) too)
(element screen
	 (make-para "Code"
		    #f))

;; sidebar - a 1x1 table
(element sidebar
	 (sosofo-append (make formatting-instruction
			      data: (string-append (parafmt "TableHolder")
						   "  &lt;Paraline
"))
			(make-table-ref)
			(make formatting-instruction
			      data: "  >
 > # end of Para
")))

;; systemitem - an object on a computer or network
(element systemitem
	 (make-string "" (case (attribute-string "role"
						 (current-node))
			       (("ftp") "SiteName")
			       (("newsgroup") "Emphasis")
			       (("url") "URL"))))

;; table - only make a reference in main mode
(element table
	 (sosofo-append (make formatting-instruction
			      data: (string-append (parafmt "TableHolder")
						   "  &lt;ParaLine 
"))
			(make-table-ref)
			(make formatting-instruction
			      data: "  >
 > # end of Para
")))

;; term - is only in variablelists so far
(element term
	 (make-para "ListVariableTerm"
		    #t))

;; title - different for different containers
(element (chapter title)
	 (sosofo-append (make formatting-instruction
			      data: (string-append (parafmt "ChapterTitle")
						   "  &lt;PgfNumString `"
						   (number->string (element-number (ancestor "CHAPTER")))
						   ".\\x11 '>
  &lt;ParaLine 
   &lt;Marker 
    &lt;MType 9>
    &lt;MText `"
						   (number->string (all-element-number (current-node)))
						   ": ChapterTitle: "))
			(with-mode text-only
				   (process-text "ChapterTitle" "text-only"))
			(make formatting-instruction
			      data: (string-append "'>
    &lt;Unique "
						   (number->string (all-element-number (current-node)))
						   ">
   > # end of Marker
   &lt;String `"))
			(process-text "ChapterTitle" "")
			(make formatting-instruction
			      data: "'>
  >
 > # end of Para
")))
(element (sect1 title)
	 (make-para "HeadA"
		    #t))
(element (sect2 title)
	 (make-para "HeadB"
		    #t))
(element (sect3 title)
	 (make-para "HeadC"
		    #t))
(element (sect4 title)
	 (make-para "HeadD"
		    #t))

;; userinput
(element userinput
	 (make-string "" "UserInput"))

;; xref - make reference to Marker
(element xref
	 (sosofo-append (make formatting-instruction
			      data: "'>
   &lt;XRef 
")
			(let ((role (attribute-string "role"
						      (current-node)))
			      (linkend (attribute-string "linkend"
							 (current-node))))
			  (case role
				(("cmd") (with-mode xref-cmd
						    (process-element-with-id linkend)))
				((#f) (with-mode xref
						 (process-element-with-id linkend)))))
			(make formatting-instruction
			      data: "'>
   &lt;Font 
    &lt;FTag `'>
    &lt;FLocked No>
   > # end of Font
   &lt;XRefEnd>
   &lt;String `")))

<external-specification id="iso-10179-procedures" document="isoprocs">
<external-specification id="mifout" document="mifout">
<external-specification id="useful" document="useful">
